@@ -31,9 +31,9 @@ impl PgBufMutExt for Vec<u8> {
     #[inline]
     fn put_statement_name(&mut self, id: u32) {
         // N.B. if you change this don't forget to update it in ../describe.rs
-        self.extend(b"sqlx_s_");
+        self.extend(b"cdbc_s_");
 
-        itoa::write(&mut *self, id).unwrap();
+        self.extend(itoa::Buffer::new().format(id).as_bytes());
 
         self.push(0);
     }
@@ -44,7 +44,7 @@ impl PgBufMutExt for Vec<u8> {
         if let Some(id) = id {
             self.extend(b"sqlx_p_");
 
-            itoa::write(&mut *self, id).unwrap();
+            self.extend(itoa::Buffer::new().format(id).as_bytes());
         }
 
         self.push(0);
